#####################################
##
## 第５章 問題解決：意思決定支援
##
##
## Author: Yohei Sato
##
#####################################


#####################################
##
## 5.1 グループ化してターゲットを決める
##
#####################################


##
## 5.1.1 主成分分析
##


# データの読み込み
sp.user.data <- read.csv("sp_user_research_data.csv", header = T)
head(sp.user.data)


# 主成分分析の実行
sp.user.pca <- prcomp(sp.user.data[, -1], scale = T)

# バイプロットの表示
biplot(sp.user.pca)

# サマリ
summary(sp.user.pca) 

##
## 5.1.2 クラスタリング
##

library(ykmeans)
args(ykmeans)

# k-meansの実行
sp.user.km <- ykmeans(
  x             = data.frame(sp.user.pca$x), 
  variable.name = paste0("PC",1:17), 
  target.name   = "PC1",
  k.list        = 4)

table(sp.user.km$cluster)


# 描画
library(ggplot2)

sp.user.km$cluster <- as.factor(sp.user.km$cluster)
ggplot(sp.user.km, aes(x = PC1, y = PC2, col = cluster, shape = cluster)) + 
  geom_point() + 
  theme_bw()



library(reshape2)

# もとのデータにクラスタ番号を追加
sp.user.data$cluster <- sp.user.km$cluster

# データリスト形式のデータに変換
sp.user.cluster.melt <- melt(sp.user.data[, -1], id.vars = "cluster")
head(sp.user.cluster.melt)

# クラスターごとに平均値を算出する
sp.user.cluster.cast <- dcast(cluster ~ variable, data = sp.user.cluster.melt, mean)
head(sp.user.cluster.cast)


# レーダーチャート用にデータを整形
df <- data.frame(scale(sp.user.cluster.cast[, -1]))
dfmax <- apply(df, 2, max) + 1
dfmin <- apply(df, 2, min) - 1
df <- rbind(dfmax, dfmin, df)

# レーダーチャートの描画
library(fmsb)

radarchart(df, seg = 5, plty = 1:4, pcol = rainbow(4), plwd = 4)
legend('topright', legend = 1:4, col = rainbow(4), lty = 1:4, lwd = 4)


#####################################
##
## 5.2 知覚マップを描いてポジショニングを決める
##
#####################################


##
## 5.2.1 コレスポンデンス分析
##

brand.image.data <- read.csv("ca_sample.csv", header=T, stringsAsFactors=F,fileEncoding="UTF-8")
head(brand.image.data)

library(reshape2)

# データリスト形式に変換
bi.data.melt <- melt(brand.image.data[,-1], id.vars="brand")
head(bi.data.melt)

# ブランド毎に各項目について集計
bi.data <- dcast(bi.data.melt, brand~variable, sum)
rownames(bi.data) <- bi.data$brand
bi.data

# コレスポンデンス分析を実行するためのパッケージ
library(ca)

# コレスポンデンス分析の実行
bi.ca <- ca(bi.data[,-1])
bi.ca

par(family="Osaka")
plot(bi.ca, labels=c(2,1))

##
## 5.2.2 多次元尺度法
##


# ブランド評価データの読み込み
bi.score.data <- read.csv("brand_score_sample.csv", header = T)
head(bi.score.data)

# 距離の算出
bi.dist <- dist(t(bi.score.data[, -1]))
bi.dist


# 非計量MDSの実行
library(MASS)
bi.mds <- isoMDS(bi.dist)

# 描画用のデータ整形
bi.mds.df <- data.frame(scale(bi.mds$points))
bi.mds.df$brand <- names(bi.score.data[, -1])
bi.mds.df


# 描画
library(ggplot2)

ggplot(bi.mds.df, aes(x = X1, y = X2, label = brand)) +
  geom_point() +
  geom_text(hjust=-0.5) + 
  theme_bw()


##
## 5.2.3 選好回帰分析
##

# 選好データの読み込み
target.data <- read.csv("target_preference_data.csv", header = T)
head(target.data)



# 非計量MDSの実行
library(MASS)
service.dist <- dist(t(target.data[, -1]))
service.map <- isoMDS(service.dist)

  
# 描画用のデータ整形
service.map.df <- data.frame(scale(service.map$points))
service.map.df$service_name <- names(target.data[, -1])
service.map.df

# 描画
library(ggplot2)

ggplot(service.map.df, aes(x = X1, y = X2, label = service_name)) +
  geom_text() + 
  theme_bw()

# melt型に変形
library(reshape2)

target.data.melt <- melt(target.data, id.vars = "id", variable.name ="service_name")
head(target.data.melt)


# 知覚マップのX1、X2を付ける
target.data.melt <- merge(target.data.melt, service.map.df, by ="service_name")
head(target.data.melt)



# 選好ベクトルの推定
library(dplyr)
library(foreach)

user.preference.data <- 
  rbind_all(foreach(id = target.data$id, combine = rbind) %do% {
    x <- target.data.melt[target.data.melt$id == id, -1]
    fit <- lm(value ~ ., data = x)
    b <- 2/sqrt(fit$coef["X1"]^2 + fit$coef["X2"]^2)
    data.frame(id = id, X1 = b * fit$coef["X1"], X2 = b * fit$coef["X2"])
    })

head(user.preference.data)


# 選好ベクトルの描画
library(ggplot2)
library(grid)

# 算出したユーザーごとの選好ベクトルの平均の算出
p.x <- mean(user.preference.data$X1)
p.y <- mean(user.preference.data$X2)

ggplot(service.map.df, aes(x = X1, y = X2)) + 
  geom_text(aes(label = service_name)) + 
  theme_bw(16) + 
  geom_point(data = user.preference.data, aes(x = X1, y = X2), shape = 3) + 
  geom_segment(aes(x = 0, y = 0, xend = p.x, yend = p.y),
               arrow = arrow(length = unit(1, "cm")), lwd = 1, col = "red")



#####################################
##
## 5.3 因果関係を把握してアクションを決める
##
#####################################

##
## 5.3.2 相関分析
##

# データの読み込み
promo.data <- read.csv("promo_data.csv", header = T, as.is = T)
head(promo.data)

# グラフカスタマイズ
library(ggplot2)
library(scales)

my_theme <- function() {
  theme_bw(base_family = "Osaka") + 
    theme(axis.text.x =element_text(angle = 45, vjust = 0.5))
}


ggplot(promo.data,aes(x=cost,y=sales)) + 
  geom_point(size=4) + 
  scale_y_continuous(label = comma) +
  scale_x_continuous(label = comma) +
  facet_grid(.~promo_type,scales="free_x") +
  xlab("コスト") +
  ylab("売上") +
  my_theme()


# 外れ値除外
promo.data <- subset(promo.data, cost<10000000)



ggplot(promo.data,aes(x=cost,y=sales)) + 
  geom_point(size=4) + 
  scale_y_continuous(label = comma) +
  scale_x_continuous(label = comma) +
  facet_grid(.~promo_type,scales="free_x") +
  xlab("コスト") +
  ylab("売上") +
  my_theme()


library(dplyr)

promo.data %>% 
  group_by(promo_type) %>% 
  summarise(cor=cor(sales, cost))


##
## 5.3.3 回帰分析
##

# GRPと売上データの読み込み
grp.data <- read.csv("grp.csv", header = T)
head(grp.data)

# 散布図の描画
ggplot(grp.data, aes(x = grp, y = amount)) + 
  geom_point() +
  scale_y_continuous(label = comma) + 
  ylab("売上") + 
  xlab("GRP") + 
  my_theme()


# 線形モデルの可視化は、geom_smooth関数ですぐに出来る
ggplot(grp.data, aes(x = grp, y = amount)) + 
  geom_point() + 
  scale_y_continuous(label = comma) + 
  ylab("売上") + 
  xlab("GRP") + 
  geom_smooth(method = "lm") + 
  my_theme()


# モデル構築
fit <- lm(amount ~ grp, data = grp.data)

# モデル概要の表示
summary(fit)



##
## 5.3.4 重回帰分析
##

ad.data <- read.csv("./ad_result.csv", header = T, stringsAsFactors = F)
head(ad.data)

# グラフ用のパッケージの読み込み
library(ggplot2)
library(scales)

# グラフカスタマイズ
my_theme <- function() {
  theme_bw(base_family = "Osaka")
}

# TVCM
ggplot(ad.data, aes(x = tvcm, y = install)) + 
  geom_point() +
  scale_x_continuous(label = comma) + 
  scale_y_continuous(label = comma) + 
  my_theme()

# 雑誌
ggplot(ad.data, aes(x = magazine, y = install)) + 
  geom_point() + 
  scale_x_continuous(label = comma) + 
  scale_y_continuous(label = comma) +
  my_theme()


# 重回帰モデルの構築
fit <- lm(install ~ ., data = ad.data[,-1])

# モデル概要
summary(fit)

# 予測値のカラムを追加
ad.data2 <- ad.data
ad.data2$fitted <- fit$fitted.values

head(ad.data2)


# グラフ用にmelt型のデータに変換
library(reshape2)

ad.data2.melt <-melt(ad.data2[,c("month","install","fitted")],id.vars="month")


# グラフの描画
ggplot(ad.data2.melt, aes(x = month, y = value, col=variable, group=variable, lty=variable, shape=variable)) + 
  geom_line() +
  geom_point() +
  scale_y_continuous(label = comma) +
  my_theme()

##
## 5.3.5 弾力性分析
##

# 対数線形型モデルの構築
fit <- lm(log(amount) ~ log(grp), data = grp.data)

# モデル概要
summary(fit)


# 予測結果のデータフレーム作成
fit.data <- data.frame(grp = grp.data$grp, amount=exp(fit$fitted.values))
head(fit.data)


ggplot(grp.data, aes(x = grp, y = amount)) + 
  geom_point() + 
  geom_line(data = fit.data, aes(x = grp, y = amount), col="blue") + 
  scale_y_continuous(label = comma) + 
  ylab("売上") + 
  xlab("GRP") + 
  my_theme()


##
## 5.3.6 ロジスティック回帰分析
##

MobilePhones <- read.csv("MobilePhones.csv", header=T, as.is=T)
head(MobilePhones)

# ロジスティック回帰モデルの構築
MobilePhones$mincost <- MobilePhones$mincost * 100
MobilePhones$sms <- MobilePhones$sms * 100
fit <- glm(choice~., data=MobilePhones, family=binomial)

# モデル概要
summary(fit)

# 正解率の算出
res <- predict(fit, type="response")
res <- ifelse(res>=0.5, 1, 0)

tt <- table(res, MobilePhones$choice)
sum(diag(tt)) / sum(tt)

# オッズ比の計算
library(epicalc)

oz <- as.data.frame(logistic.display(fit, simplified=T)$table)

# オッズ比の逆数をつける
oz$OR.inverse <- 1 / oz$OR

# 表示
oz


#####################################
##
## 5.4 パターンやルールを見つけて施策を決める
##
#####################################

##
## 5.4.1 アソシエーション分析
##

library(arules)

data(Groceries)
Groceries
summary(Groceries)

# 行列に変換
mtx <- as(Groceries, "matrix")
head(mtx[,1:7])

# アイテムの出現頻度
itemFrequencyPlot(Groceries, horiz = T)

# アソシエーションルールの抽出
## デフォルトで実行
groceries.ap <- apriori(Groceries)


## サポートとコンフィデンスの閾値を変更して実行
groceries.ap <- apriori(Groceries, parameter = list(support = 0.001,confidence = 0.001, 
                                                    minlen = 2, maxlen=2))

# supportでソート
inspect(head(sort(groceries.ap, by = "support"), 3))

# confidenceでソート
inspect(head(sort(groceries.ap, by = "confidence"), 3))

# liftでソート
inspect(head(sort(groceries.ap, by = "lift"), 3))

# ルール長を3で実行
groceries.ap <- apriori(Groceries, parameter = list(support = 0.001,confidence = 0.001, 
                                                    minlen = 3, maxlen = 3))
# liftでソート
inspect(head(sort(groceries.ap, by = "lift"), 3))

# アソシエーションルールの可視化
library(arulesViz)

plot(groceries.ap)
plot(groceries.ap, measure = c("support", "lift"), shading = "confidence")

plot(subset(groceries.ap,subset=lift<1))
plot(subset(groceries.ap,subset=lift>=1))
plot(subset(groceries.ap,subset=lift>=10))

# ルールのグループ化
set.seed(10)
plot(groceries.ap, method = 'grouped', control = list(k =10))

# ルールのグループ化（lift>=10）
set.seed(10)
plot(subset(groceries.ap,subset=lift>=10), method = 'grouped', control = list(k =10))
