#####################################
##
## 第６章 問題解決：自動化・最適化
##
##
## Author: Yohei Sato
##
#####################################


# データの読み込み
juyo.data <- read.csv("juyo_data.csv", header=T, as.is=T)
head(juyo.data)


# 規則性とまとまりの検証
library(ggplot2)
library(scales)

juyo.data$log_date <- as.Date(juyo.data$log_date)

ggplot(juyo.data, aes(x=log_date, y=amount)) +
  geom_line() +
  geom_point() +
  scale_y_continuous(label=comma) +
  xlab("") +
  theme_bw()


# 説明変数の作成
x <- juyo.data
names(x)[2] <- "p_0"

for (i in 1:8) {
  y <- juyo.data
  y$log_date <- y$log_date+i
  names(y)[2] <- sprintf("p_%s", i)  
  x <- merge(x, y, by="log_date")
}

# 特徴量の無相関化
x.pca <- as.data.frame(prcomp(x[,-1], scale=T)$x)
x.pca$log_date <- x$log_date

# クラスタリング
library(ykmeans)

x.km <- ykmeans(x.pca, variable.names=paste0("PC",1:8), 
                target.name="PC1", k.list=4:6, n=100)

library(dplyr)
x <- inner_join(x, x.km[,c("log_date","cluster")], by="log_date")
head(x)



table(x$cluster)
table(substr(x$log_date,6,7), x$cluster)


library(reshape2)

# クラスタ毎に平均を算出
x.melt0 <- melt(x, id.vars="cluster", measure.vars=paste0("p_",1:8))
x.cast <- dcast(x.melt0, cluster~variable, mean)

# データリスト形式に変換
x.melt <- melt(x.cast, id.vars="cluster")

# クラスタIDをファクター型に変換
x.melt$cluster <- as.factor(x.melt$cluster)


# 特徴をプロット
ggplot(x.melt, aes(x=variable, y=value, col=cluster, group=cluster, shape=cluster, lty=cluster)) +
  geom_line() +
  geom_point() +
  scale_y_continuous(label=comma) +
  theme_bw()


# 1年単位のデータを複数個作成
term <- 365
train.list <- lapply((term+2):nrow(x), function(i){
  x[(i-(term+1)):i,]
})


# 1つのデータセットで予測モデルを構築して比較する
x1 <- train.list[[1]]

# モデルを構築するために使うデータ
x1.train <- x1[-nrow(x1),]

# テスト用データ
x1.test <- x1[nrow(x1),]

# 特徴量の無相関化
x1.pca <- prcomp(x1.train[,4:10], scale=T)
x.pc <- as.data.frame(x1.pca$x)
x.pc$p_1 <- x1.train$p_1
x.pc$p_0 <- x1.train$p_0

plot(x.pc[, paste0("PC",1:7)])


library(caret)

# 学習方法のカスタマイズ
fit.control <- trainControl(method = "CV", p = 0.75, number = 30)

##
## 線形回帰モデル
##
fit.lm <- train(x=x.pc[,1:7], y=x1.train$p_1,
                method="lm", trControl = fit.control)

fit.lm$result

# 使用出来るモデルとパラメータ一覧
model.list <- modelLookup()
head(model.list, 10)

# 使用出来るモデル一覧
unique(model.list$model)


##
## ランダムフォレスト
##

fit.rf <- train(x=x.pc[,1:7], y=x1.train$p_1,
                method="rf", trControl = fit.control)
fit.rf$result

##
## SVM回帰
##

## SVM回帰 (線形カーネル)
fit.svm1 <- train(x=x.pc[,1:7], y=x1.train$p_1,
                  method="svmLinear", trControl = fit.control)

library(dplyr)

fit.svm1$result %>% arrange(RMSE) %>% head()

## SVM回帰 (多項式カーネル)
fit.svm2 <- train(x=x.pc[,1:7], y=x1.train$p_1,
                  method="svmPoly", trControl = fit.control)

fit.svm2$result %>% arrange(RMSE) %>% head()


## SVM回帰 (ガウジアンカーネル)
fit.svm3 <- train(x=x.pc[,1:7], y=x1.train$p_1,
                  method="svmRadial", trControl = fit.control)

fit.svm3$result %>% arrange(RMSE) %>% head()


##
## モデルの比較
##

library(foreach)

model.list <- list(
  "lm", # 線形回帰
  "rf",        # ランファムフォレスト
  "svmLinear", # SVM (線形カーネル)
  "svmPoly",   # SVM (多項式カーネル)
  "svmRadial"  # SVM (ガウジアンカーネル)
)

fit.list <- foreach(model=model.list, combine=rbind) %do%{
  train(x=x.pc[,1:7], y=x1.train$p_1,
        method=model, trControl = fit.control)
}

names(fit.list) <- model.list


# 各モデルの結果サマリ
fit.df <- rbind_all(foreach(model=model.list, combine=rbind) %do% {
  y <- x.pc[-1,]
  y.act <- x1.train$p_0[-nrow(x1.train)]
  y.pred <- predict(fit.list[[model]], newdata=y[,1:7])
  data.frame(
    model      = model,
    train.cor  = cor(x1.train$p_1, predict(fit.list[[model]])),
    test.cor   = cor(y.act, y.pred),
    train.RMSE = min(fit.list[[model]]$results$RMSE),
    test.RMSE  = sqrt(mean((y.act-y.pred)^2))
  )
})

fit.df


# 検証データとの誤差が一番小さいモデルを選定
best.model <- fit.list[[as.character(arrange(fit.df, test.RMSE)$model[1])]]

# テスト用データを変換 (無相関化された特徴空間に写像する)
x1.test.pc <- as.matrix((x1.test[1,4:10] - x1.pca$center) / x1.pca$scale) %*% x1.pca$rotation

# 予測
x1.test$pred <- predict(best.model, newdata=x1.test.pc)
x1.test


##
## 予測モデルの自動選択を実装する
##

result <- rbind_all(foreach(i=1:length(train.list), combine=rbind) %do%{
  x1 <- train.list[[i]]
  
  # 学習用とテスト用データに分ける
  x1.train <- x1[-nrow(x1),]
  x1.test <- x1[nrow(x1),]
  
  # 特徴量の無相関化
  x1.pca <- prcomp(x1.train[,4:10], scale=T)
  x.pc <- as.data.frame(x1.pca$x)
  x.pc$p_1 <- x1.train$p_1
  x.pc$p_0 <- x1.train$p_0
  
  # 予測モデルの構築
  fit.list <- foreach(model=model.list, combine=rbind) %do%{
    train(x=x.pc[,1:7], y=x1.train$p_1,
          method=model, trControl = fit.control)
  }
  
  names(fit.list) <- model.list
  
  # 各モデルの結果サマリ
  fit.df <- rbind_all(foreach(model=model.list, combine=rbind) %do% {
    y <- x.pc[-1,]
    y.act <- x1.train$p_0[-nrow(x1.train)]
    y.pred <- predict(fit.list[[model]], newdata=y[,1:7])
    data.frame(
      model      = model,
      train.cor  = cor(x1.train$p_1, predict(fit.list[[model]])),
      test.cor   = cor(y.act, y.pred),
      train.RMSE = min(fit.list[[model]]$results$RMSE),
      test.RMSE  = sqrt(mean((y.act-y.pred)^2))
    )
  })
  
  # 検証データとの誤差が一番小さいモデルを選定
  best.df <- fit.df %>% arrange(test.RMSE) %>% head(1)
  best.model <- fit.list[[as.character(best.df$model)]]
  
  # テストデータを変換
  x1.test.pc <- as.matrix((x1.test[1,4:10] - x1.pca$center) / x1.pca$scale) %*% x1.pca$rotation
  
  # 予測
  x1.test$model <- as.character(best.df$model)
  x1.test$pred <- predict(best.model, newdata=x1.test.pc)
  x1.test$sdev <- best.df$test.sdev
  x1.test$pred.u95 <- x1.test$pred + 3*best.df$test.sdev
  x1.test$pred.l95 <- x1.test$pred - 3*best.df$test.sdev
  x1.test
})


result.melt <- melt(result, id.vars="log_date", measure.vars=c("p_0", "pred"))

ggplot(result.melt, aes(x=log_date, y=value, col=variable, lty=variable, shape=variable)) +
  geom_line() +
  geom_point() +
  scale_y_continuous(label=comma) +
  theme_bw()


# 異常度スコアの算出
result$d.score <- (result$pred - result$p_0) / result$sdev

ggplot(result, aes(x=log_date, y=d.score)) +
  geom_line() +
  geom_point() +
  scale_y_continuous(label=comma) +
  theme_bw()


