#####################################
##
## 第７章 効果測定
##
##
## Author: Yohei Sato
##
#####################################



#####################################
##
## 7.2 ランダムなグループ間で比較する
##
#####################################

##
## A/Bテスト
##

ab.test.data <- read.csv("ab_test_sample.csv", header=T, stringsAsFactors=F)
head(ab.test.data)

library(reshape2)

ab.test.data.cast <- dcast(ab.test.data, day~group, value.var="cv",mean)
ab.test.data.cast


ab.test.data.melt <- melt(ab.test.data.cast, id.vars="day")

library(ggplot2)
library(scales)

ggplot(ab.test.data.melt, aes(x=day, y=value, col=variable, group=variable, lty=variable, shape=variable)) +
  geom_line() +
  geom_point() +
  scale_y_continuous(label=percent) +
  xlab("") +
  ylab("CVR") +
  theme_bw()

# 実施前と実施後に分ける
ab.test.data.cast1 <- subset(ab.test.data.cast, day<="04-07")
ab.test.data.cast2 <- subset(ab.test.data.cast, day>="04-08")

# それぞれ効果算出
ab.test.effect1 <- mean(ab.test.data.cast1$a - ab.test.data.cast1$b) 
ab.test.effect2 <- mean(ab.test.data.cast2$a - ab.test.data.cast2$b)

ab.test.effect1 # ほぼ0のはず
ab.test.effect2

# 検定
t.test(ab.test.data.cast1$a, ab.test.data.cast1$b)
t.test(ab.test.data.cast2$a, ab.test.data.cast2$b)


##
## 多変量テスト
##

library(conjoint)

# 構成要素
experiment <- expand.grid(
  imgA = c("ImageA1", "ImageA2"), 
  imgB = c("ImageB1", "ImageB2"), 
  txtA = c("TextA1", "TextA2"), 
  txtB = c("TextB1", "TextB2"))

# テストパターンの作成
design.ort <- caFactorialDesign(data = experiment, type = "orthogonal")
design.ort

# 仮想のテスト結果データの読み込み
web.test.data <- read.csv("web_test_sample.csv", header = T)
head(web.test.data)


# ロジスティック回帰モデルの構築
fit <- step(glm(cv ~ ., data = web.test.data, family = binomial))

# モデル概要
summary(fit)


library(epicalc)

logistic.display(fit, simplified = T)

oz.df <- as.data.frame(logistic.display(fit, simplified = T)$table)
oz.df$OR.Receiprocal <- 1 / oz.df$OR
oz.df


#####################################
##
## 7.3 非ランダムなグループ間で比較する
##
#####################################


##
## 層別解析
##

shisaku.effect <- read.csv("shisaku_effect.csv", header=T, as.is=T)
head(shisaku.effect)

library(dplyr)
library(ggplot2)
library(scales)

shisaku.effect$log_date <- as.Date(shisaku.effect$log_date)

shisaku.effect %>%
  group_by(log_date, group) %>%
  summarise(arpu=mean(payment)) %>%
  ggplot(aes(x=log_date, y=arpu, col=group)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("ARPU") +
  scale_y_continuous(label=comma) +
  theme_bw()


uu.data <- shisaku.effect %>%
  group_by(log_date, group) %>%
  summarise(uu=length(payment)) 

ylimits <- c(0, max(uu.data$uu))

ggplot(uu.data, aes(x=log_date, y=uu, col=group, lty=group, shape=group)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("UU") +
  scale_y_continuous(label=comma, limits=ylimits) +
  theme_bw()



shisaku.effect %>%
  group_by(log_date, group, gender) %>%
  summarise(uu=length(payment)) %>%
  ggplot(aes(x=log_date, y=uu, fill=gender,group=gender)) +
  geom_bar(stat="identity", col="black") +
  xlab("") +
  ylab("UU") +
  scale_y_continuous(label=comma) +
  facet_grid(group~.) +
  theme_bw() +
  scale_fill_brewer(palette = "Paired")


shisaku.effect %>%
  group_by(log_date, group, gender) %>%
  summarise(arpu=mean(payment)) %>%
  ggplot(aes(x=log_date, y=arpu, col=group)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("ARPU") +
  scale_y_continuous(label=comma) +
  theme_bw() +
  facet_grid(gender~., scale="free") +
  theme(legend.position="top")


# 男女別の施策効果を計算
shisaku.effect.cast <- 
  shisaku.effect %>%
  filter(log_date>=as.Date("2014-05-06")) %>%
  group_by(group, gender) %>%
  summarise(arpu=mean(payment)) %>%
  dcast(gender~group, mean, value.var="arpu") %>%
  mutate(effect=B-A)

# 男女別の施策効果
shisaku.effect.cast

# 全体の施策効果 (1日平均)
mean(shisaku.effect.cast$effect) 

##
## 傾向スコア解析
##

nsw.data <- read.csv("nsw.csv", header=T)
head(nsw.data)

# 単純な平均差
nsw0.treat <- mean(subset(nsw.data, treat==1)$re78)
nsw0.ctrl <- mean(subset(nsw.data,treat==0)$re78)
nsw0.treat - nsw0.ctrl

# 傾向スコア
logit <- glm(treat~., data=nsw.data[,-10], family=binomial)
summary(logit)


# マッチングによる効果測定
library(Matching)

nsw1 <- Match(Y=nsw.data$re78, Tr=nsw.data$treat, X=logit$fitted)
summary(nsw1)


# IPW推定量による効果測定
ivec1 <- nsw.data$treat
ivec2 <- rep(1, nrow(nsw.data)) - ivec1
iestp1 <- (ivec1/logit$fitted) * (length(ivec1)/sum(ivec1))
iestp2 <- (ivec2/(1-logit$fitted)) * (length(ivec2)/sum(ivec2))
iestp <- iestp1 + iestp2

ivec <- cbind(ivec1, ivec2)
ipwe <- lm(nsw.data$re78~ivec-1, weights=iestp)
summary(ipwe)

# 効果の算出
coef(ipwe)[1] - coef(ipwe)[2]

# 効果の標準誤差
treat.error <- summary(ipwe)$coef[1,2]
ctrl.error <- summary(ipwe)$coef[2,2]
sqrt(treat.error^2+ctrl.error^2)
